package com.ibm.gre.samples.collectionhelpers;

import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.util.HashMap;

import javax.ejb.EJBObject;
import javax.ejb.RemoveException;

import com.ibm.gre.engine.ejb.facade.BatchControllerEJB;
import com.ibm.gre.engine.ejb.facade.ClientControllerEJB;
import com.ibm.gre.engine.ejb.facade.ComponentDesignControllerEJB;
import com.ibm.gre.engine.ejb.facade.ConfigurationControllerEJB;
import com.ibm.gre.engine.ejb.facade.DefinitionControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentContentControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanDesignControllerEJB;
import com.ibm.gre.engine.ejb.facade.ImportExportControllerEJB;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJB;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJB;
import com.ibm.gre.engine.ejb.facade.QueryControllerEJB;
import com.ibm.gre.engine.ejb.facade.RecordsOperationsControllerEJB;
import com.ibm.gre.engine.ejb.facade.SecurityControllerEJB;
import com.ibm.gre.engine.ejb.facade.SecurityManagementControllerEJB;
import com.ibm.gre.engine.ejb.facade.TemplateControllerEJB;
import com.ibm.gre.engine.ejb.facade.ToolsControllerEJB;
import com.ibm.gre.engine.ejb.facade.UserControllerEJB;
import com.ibm.gre.engine.util.RecordManagerException;

/**
 * Keeps track of the client controller and other controllers.
 * Also contains convenience methods for logging in to the IRM engine.
 */
public class ClientSession {
	
	private ClientControllerEJB clientController;
	
	private HashMap controllers; 

	/**
	 * Constructor for Client.
	 */
	public ClientSession(ClientControllerEJB clientController) {
		super();
		
		this.clientController = clientController;
		this.controllers = new HashMap(16);
	}

	/**
	 * Get a cached controller. If their is no controller in the cache, one is added.
	 */
	private Object getController(Class controllerClass)
		throws RecordManagerException, RemoteException {
			
		if (!controllers.containsKey(controllerClass))
			loadController(controllerClass);
		return controllers.get(controllerClass);
	}
	
	/**
	 * Returns a new controller. If there is an old controller in the cache it is replaced.
	 */
	private Object getNewController(Class controllerClass)
		throws RecordManagerException, RemoteException {

		if (controllers.containsKey(controllerClass)) {
			try {
				((EJBObject)controllers.get(controllerClass)).remove();
			} catch (Exception e) {
			}
		}
		loadController(controllerClass);
		return controllers.get(controllerClass);
	}
	
	/**
	 * Loads a controller into the cache. 
	 */
	private void loadController(Object key) throws RemoteException, RecordManagerException {
		try {
			if (FilePlanDesignControllerEJB.class.equals(key)) {
				controllers.put(
					FilePlanDesignControllerEJB.class,
					clientController.getFilePlanDesignController());
				return;
			}
			if(ComponentDesignControllerEJB.class.equals(key)){
				controllers.put(ComponentDesignControllerEJB.class, clientController.getComponentDesignController());
				return;
			}
			if(ConfigurationControllerEJB.class.equals(key)){
				controllers.put(ConfigurationControllerEJB.class, clientController.getConfigurationController());
				return;
			}	
			if(FilePlanComponentControllerEJB.class.equals(key)){
				controllers.put(FilePlanComponentControllerEJB.class, clientController.getFilePlanComponentController());
				return;
			}	
			if(ToolsControllerEJB.class.equals(key)){
				controllers.put(ToolsControllerEJB.class, clientController.getToolsController());
				return;
			}			
			if(TemplateControllerEJB.class.equals(key)){
				controllers.put(TemplateControllerEJB.class, clientController.getTemplateController());
				return;
			}	
			if(DefinitionControllerEJB.class.equals(key)){
				controllers.put(DefinitionControllerEJB.class, clientController.getDefinitionController());
				return;
			}
			if(UserControllerEJB.class.equals(key)){
				controllers.put(UserControllerEJB.class, clientController.getUserController());
				return;
			}
			if(QueryControllerEJB.class.equals(key)){
				controllers.put(QueryControllerEJB.class, clientController.getQueryController());
				return;
			}
			if (LifeCycleDesignControllerEJB.class.equals(key)) {
				controllers.put(LifeCycleDesignControllerEJB.class, clientController.getLifeCycleDesignController());
				return;
			}
			if (RecordsOperationsControllerEJB.class.equals(key)) {
				controllers.put(RecordsOperationsControllerEJB.class, clientController.getRecordsOperationsController());
				return;
			}
			if(BatchControllerEJB.class.equals(key)){
				controllers.put(BatchControllerEJB.class, clientController.getBatchController());
				return;
			}
			if(NavigationControllerEJB.class.equals(key)){
				controllers.put(NavigationControllerEJB.class, clientController.getNavigationController());
				return;
			}
			if(FilePlanComponentContentControllerEJB.class.equals(key)){
				controllers.put(FilePlanComponentContentControllerEJB.class,clientController.getFilePlanComponentContentController());
				return;
			}
			if(SecurityControllerEJB.class.equals(key)){
				controllers.put(SecurityControllerEJB.class,clientController.getSecurityController());
				return;
			}
			if(SecurityManagementControllerEJB.class.equals(key)){
				controllers.put(SecurityManagementControllerEJB.class,clientController.getSecurityManagementController());
				return;
			}
			if(ImportExportControllerEJB.class.equals(key)){
				controllers.put(ImportExportControllerEJB.class,clientController.getImportExportController());
				return;
			}
			
		}
		catch (NoSuchObjectException e) {
			// the clientController is invalid, may be a time out or server problem
			clientController = null;
			controllers.clear();
			throw new RuntimeException("Your session has expired.");					
		}
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getComponentDesignController()
	 */
	public BatchControllerEJB getBatchController()
		throws RecordManagerException, RemoteException {
		return (BatchControllerEJB) getController(BatchControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getComponentDesignController()
	 */
	public ComponentDesignControllerEJB getComponentDesignController()
		throws RecordManagerException, RemoteException {
		return (ComponentDesignControllerEJB) getController(ComponentDesignControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getConfigurationController()
	 */
	public ConfigurationControllerEJB getConfigurationController()
		throws RecordManagerException, RemoteException {
			
		return (ConfigurationControllerEJB) getController(ConfigurationControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getDefinitionController()
	 */
	public DefinitionControllerEJB getDefinitionController()
		throws RecordManagerException, RemoteException {

		return (DefinitionControllerEJB) getController(DefinitionControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getFilePlanComponentController()
	 */
	public FilePlanComponentControllerEJB getFilePlanComponentController()
		throws RecordManagerException, RemoteException {
			
		return (FilePlanComponentControllerEJB) getController(FilePlanComponentControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getFilePlanDesignController()
	 */
	public FilePlanDesignControllerEJB getFilePlanDesignController()
		throws RecordManagerException, RemoteException {

		return (FilePlanDesignControllerEJB) getController(FilePlanDesignControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getLifeCycleDesignController()
	 */
	public LifeCycleDesignControllerEJB getLifeCycleDesignController()
		throws RecordManagerException, RemoteException {

		return (LifeCycleDesignControllerEJB) getController(LifeCycleDesignControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getQueryController()
	 */
	public QueryControllerEJB getQueryController()
		throws RecordManagerException, RemoteException {

		return (QueryControllerEJB) getController(QueryControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getRecordsOperationsController()
	 */
	public RecordsOperationsControllerEJB getRecordsOperationsController()
		throws RecordManagerException, RemoteException {

		return (RecordsOperationsControllerEJB) getController(RecordsOperationsControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getTemplateController()
	 */
	public TemplateControllerEJB getTemplateController()
		throws RecordManagerException, RemoteException {

		return (TemplateControllerEJB) getController(TemplateControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getToolsController()
	 */
	public ToolsControllerEJB getToolsController()
		throws RecordManagerException, RemoteException {

		return (ToolsControllerEJB) getController(ToolsControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getUserController()
	 */
	public UserControllerEJB getUserController()
		throws RecordManagerException, RemoteException {

		return (UserControllerEJB) getController(UserControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getNavigationController()
	 */
	public NavigationControllerEJB getNavigationController()
		throws RecordManagerException, RemoteException {
			
			return (NavigationControllerEJB) getController(NavigationControllerEJB.class);
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getFilePlanComponentContentController()
	 */
	public FilePlanComponentContentControllerEJB getFilePlanComponentContentController()
		throws RecordManagerException, RemoteException {
	
		return (FilePlanComponentContentControllerEJB) getController(FilePlanComponentContentControllerEJB.class);			
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getSecurityController()
	 */
	public SecurityControllerEJB getSecurityController()
		throws RecordManagerException, RemoteException {
	
		return (SecurityControllerEJB) getController(SecurityControllerEJB.class);			
	}
	
	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getSecurityManagementController()
	 */
	public SecurityManagementControllerEJB getSecurityManagementController()
		throws RecordManagerException, RemoteException {
	
		return (SecurityManagementControllerEJB) getController(SecurityManagementControllerEJB.class);			
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#getSecurityManagementController()
	 */
	public ImportExportControllerEJB getImportExportController()
		throws RecordManagerException, RemoteException {
	
		return (ImportExportControllerEJB) getController(ImportExportControllerEJB.class);			
	}

	/**
	 * @see com.ibm.gre.engine.ejb.facade.ClientControllerEJB#logout()
	 */
	public void logout() throws RemoteException {
		if (getClientController() == null)
			return;
		getClientController().logout();
		try {
			getClientController().remove();
			setClientController(null);
		} catch (RemoveException e) {
		}
	}

	/**
	 * Returns the clientController.
	 * @return ClientControllerEJB
	 */
	public ClientControllerEJB getClientController() {
		return clientController;
	}

	/**
	 * Sets the clientController.
	 * @param clientController The clientController to set
	 */
	private void setClientController(ClientControllerEJB clientController) {
		this.clientController = clientController;
	}
	

}
