/*
 * Created on Jan 26, 2005
 */
package com.ibm.gre.samples.collectionhelpers;

import java.rmi.RemoteException;
import java.sql.Timestamp;
import java.util.Date;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.ibm.gre.engine.types.LogTypes;
import com.ibm.gre.engine.types.QueryTypes;
import com.ibm.gre.engine.util.AsynchronousTaskType;
import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.util.BatchResultStatusTypes;
import com.ibm.gre.engine.util.ConversionUtil;
import com.ibm.gre.engine.util.ProfileControlType;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.XMLUtil;

/**
 * The IRMBatchQuery class encapsulates the XML used to
 * execute a Batch Life Cycle Query.
 * There are methods for begininning the Asynchronous task
 * and subsequent methods for determining the status and getting
 * return values.
 * See the documentation for the BatchControllerEJB interface in
 * the com.ibm.gre.engine.ejb.facade package conatined in
 * the IBM DB2 Records Manager API Reference. 
 */
public class IRMBatchQuery extends IRMQuery {
	
	public IRMBatchQuery(String queryXML, ClientSession session,String strScope , boolean addDisplayInfo, int optionalFieldsMask)
	throws RecordManagerException, Exception
	{
		super(queryXML, session, strScope , addDisplayInfo);
		
		m_optionalFields = new OptionalFields(optionalFieldsMask);
		m_nStatus = (int)BatchResultStatusTypes.RECEIVED;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.gre.samples.dataexplorer.rbc.IRMQuery#Initialize()
	 */
	public void Initialize() throws Exception
	{
		//Add optional Life Cycle Fields if this is a Life Cycle Query
		if(m_nQueryType == QueryTypes.LIFE_CYCLE)
		{
			//TODO the captions should be more carefully constructed
			Element lm = (Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"GetEligibleFilePlanComponents");
			if(m_optionalFields.editReferenceDateFrom())
			{
				AddOptionalField(lm,AttributeDataTypes.DATE,"ReferenceDateFrom","ReferenceDate is From",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editReferenceDateTo())
			{
				String captn = "To";
				if(!m_optionalFields.editReferenceDateFrom())
				{
					captn = "ReferenceDate Is";
				}
				AddOptionalField(lm,AttributeDataTypes.DATE,"ReferenceDateTo",captn,ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editDisposalAuthority())
			{
				AddOptionalField(lm,AttributeDataTypes.BOOLEAN,"IncludeDisposalAuthority","IncludeDisposalAuthority",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editEligibleForCutOff())
			{
				AddOptionalField(lm,AttributeDataTypes.BOOLEAN,"EligibleForCutoff","EligibleForCutoff",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editEligibleForClose())
			{
				AddOptionalField(lm,AttributeDataTypes.BOOLEAN,"EligibleForClose","EligibleForClose",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editEligibleForTransition())
			{
				AddOptionalField(lm,AttributeDataTypes.BOOLEAN,"EligibleForTransition","EligibleForTransition",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editPhaseId())
			{
				AddOptionalField(lm,AttributeDataTypes.INTEGER,"PhaseId","PhaseId",ProfileControlType.SINGLELINE);
			}
			if(m_optionalFields.editLifeCycleQueryType())
			{
				AddOptionalField(lm,AttributeDataTypes.INTEGER,"LifeCycleQueryType","LifeCycleQueryType",ProfileControlType.SINGLELINE);
			}	
		}
			
		//Then initialize the base so that the new fields are always first
		super.Initialize();
	}

	/**
	 * Used internally in the Initialize method to add optional fields
	 * @param lm The XML element for the field.
	 * @param dataType data type.
	 * @param key The name of the element used in the GetEligibleFilePlanComponents element of the Query XML.
	 * @param caption Caption
	 * @param nControlType Type of control to use.
	 */
	private void AddOptionalField(Element lm, int dataType, String key, String caption, int nControlType)
	{
		Element fieldElement = (Element)XMLUtil.getNodeByXPath(lm,key);
		Line ln = new Line(fieldElement,dataType,key,caption,0,LineType.NONE,nControlType,m_tab);
		m_fieldvalues.put(key,ln);
		if(m_addDisplayInfo)
		{
			m_DisplayItems.add(ln);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.samples.dataexplorer.rbc.IRMQuery#runQuery(int, int)
	 */
	/**
	 *  Overrides the IRMQuery runQuery method.
	 *  If the Asynchronous task has not begun this method will
	 *  call the BatchController beginTask method to start the
	 *  Life Cycle Query.
	 *  @param nBlockStart The index value return a block of values from. 0 indicates the first value.
	 *  @param nBlockSize The number of values to return. 0 indicates return all values.
	 *  @return An integer value indicating the status of the task.
	 *  Positive values indicate percent complete.
	 *  Negative values refer to BatchResultStatusTypes.
	 */
	public int runQuery(int nBlockStart, int nBlockSize) throws Exception
	{
		super.setBlockStart(nBlockStart);
		super.setBlockSize(nBlockSize);
		if(m_strTaskKey == null)
		{
			m_strTaskName = "Get Elegible " + super.getFlPlnCmpntDefName() + "s " + now();
			m_strTaskKey = m_session.getBatchController().beginTask
				(
					AsynchronousTaskType.GET_ELIGIBLE_FILE_PLAN_COMPONENTS,
					super.getModifiedQuery(),
					m_strTaskName,
					""
				);
		}
		reLoad();
		return m_nStatus;
	}

	/**
	 * The ID of the current task.
	 * @return String
	 */
	public String getTaskId()
	{
		return m_strTaskKey;
	}

	/**
	 * The name of the current task
	 * @return String
	 */
	public String getTaskName()
	{
		return m_strTaskName;
	}

	/**
	 * Calls the BatchController getStatusListByTaskId method and extracts relevant
	 * values from the returned xml String.
	 * @throws RemoteException
	 * @throws RecordManagerException
	 */
	private void reLoad() throws RemoteException, RecordManagerException 
	{
		String xml = m_session.getBatchController().getStatusListByTaskId(m_strTaskKey);
		Document doc = XMLUtil.getDocument(xml);
		Element root = doc.getDocumentElement();
		NodeList list = root.getElementsByTagName("Item");
			
		if(list.getLength() > 0)
		{
			Element item = (Element)list.item(list.getLength() - 1);

			m_strStartTime = item.getAttributes().getNamedItem("StartTime").getNodeValue();
			m_nStatus = Integer.parseInt(item.getAttributes().getNamedItem("Status").getNodeValue());
			m_strStatusTime = item.getAttributes().getNamedItem("StatusTime").getNodeValue();
			m_strTaskID = item.getAttributes().getNamedItem("TaskID").getNodeValue();
			m_strTaskName = item.getAttributes().getNamedItem("TaskName").getNodeValue();
			m_nType = Integer.parseInt(item.getAttributes().getNamedItem("Type").getNodeValue());
			m_nUserID = Long.parseLong(item.getAttributes().getNamedItem("UserID").getNodeValue());

			m_totalResults = 0;
			if(m_nStatus == 100)
			{
				m_totalResults = Integer.parseInt(item.getAttributes().getNamedItem("ResultCount").getNodeValue());
				m_strResultList = m_session.getBatchController().getTaskResultsByID(m_strTaskID,LogTypes.RESULT_LOG,super.getBlockStart(),super.getBlockSize());
			}
		}
	}
	
	private String now()
	{
		Date d = new Date();
		Timestamp t = new Timestamp(d.getTime());
		return ConversionUtil.timestampToString(t);
	}
	
	private OptionalFields m_optionalFields;
	private String m_strTaskKey = null;

	private String m_strStartTime;
	private int m_nStatus;
	private String m_strStatusTime;
	private String m_strTaskID;
	private String m_strTaskName;
	private int m_nType;
	private long m_nUserID;
}
